function interp_abcor,star,line,grid
    ;---------------------------------------------------------------------------
    ; name:
    ;   interp_star
    ; purpose:
    ;   interpolate f[grid] onto stellar parameters {star}
    ; comments:
    ;   1. routine could be optimised by not interpolating across entire grid
    ;   
    ;   2. stellar parameters are teff, lgg, feh, vturb_base, vturb_top, lgeps:
    ;       - for 3D non-LTE versus 1D LTE corrections, vturb_base is the 1D LTE
    ;       microturbulence, vturb_top could be set to 0, and lgeps is the 
    ;       absolute 1D LTE abundance
    ;       - for Fe2, [Fe/H] is not used (it may be set to any value), but
    ;       lgeps is used
    ;   
    ;   3. extrapolation:
    ;       edge value for teff/lgg/feh/vturb_base/vturb_top
    ;       linear extrapolation for lgeps
    ;---------------------------------------------------------------------------
    lowval=-1d20
    minval=-1d29

    index=where(strmatch(grid.line,'*'+strtrim(line,2)+'*'),/null,count)
    if count eq 0l then begin
        print,'Line '+strtrim(line,2)+' unrecognized'
        stop
    endif else iline=index[0l]

    ; initialize result
    f0=grid.abcor[*,*,*,*,*,*,iline]

    ; microturbulence_top
    f=dblarr(grid.nlgeps,grid.nteff,grid.nlgg,grid.nfeh,grid.nvturb_base)
    f[*,*,*,*,*]=minval
    for ivturb=0l,grid.nvturb_base-1l do begin
        for ifeh=0l,grid.nfeh-1l do begin
            for ilgg=0l,grid.nlgg-1l do begin
                for iteff=0l,grid.nteff-1l do begin
                    for ilgeps=0l,grid.nlgeps-1l do begin
                        y=f0[ilgeps,iteff,ilgg,ifeh,ivturb,*]
                        y=reform(y)
                        x=grid.vturb_top
                        x0=star.vturb_top

                        index=where(y gt lowval,/null,count)
                        if count eq 0l then continue
                        y=y[index]
                        x=x[index]

                        ; can't interpolate?
                        if (x[0] gt x0 or x[-1] lt x0) then begin
                            ; edge value
                            if (x[0] gt x0) then a=y[0] else a=y[-1]
                        endif else a=interpol_wrap(y,x,x0,/spline,/quad)
                        f[ilgeps,iteff,ilgg,ifeh,ivturb]=a
                    endfor ; lgeps
                endfor ; teff
            endfor ; lgg
        endfor ; feh
    endfor ; vturb_base
    f0=f

    ; microturbulence_base
    f=dblarr(grid.nlgeps,grid.nteff,grid.nlgg,grid.nfeh)
    f[*,*,*,*]=minval
    for ifeh=0l,grid.nfeh-1l do begin
        for ilgg=0l,grid.nlgg-1l do begin
            for iteff=0l,grid.nteff-1l do begin
                for ilgeps=0l,grid.nlgeps-1l do begin
                    y=f0[ilgeps,iteff,ilgg,ifeh,*]
                    y=reform(y)
                    x=grid.vturb_base
                    x0=star.vturb_base
                        
                    index=where(y gt lowval,/null,count)
                    if count eq 0l then continue
                    y=y[index]
                    x=x[index]

                    ; can't interpolate?
                    if (x[0] gt x0 or x[-1] lt x0) then begin
                        ; edge value
                        if (x[0] gt x0) then a=y[0] else a=y[-1]
                    endif else a=interpol_wrap(y,x,x0,/spline,/quad)
                    f[ilgeps,iteff,ilgg,ifeh]=a
                endfor ; lgeps
            endfor ; teff
        endfor ; lgg
    endfor ; feh
    f0=f

    ; [Fe/H]
    f=dblarr(grid.nlgeps,grid.nteff,grid.nlgg)
    f[*,*,*]=minval
    for ilgg=0l,grid.nlgg-1l do begin
        for iteff=0l,grid.nteff-1l do begin
            for ilgeps=0l,grid.nlgeps-1l do begin
                y=f0[ilgeps,iteff,ilgg,*]
                y=reform(y)
                x=grid.feh
                x0=star.feh
                    
                index=where(y gt lowval,/null,count)
                if count eq 0l then continue
                y=y[index]
                x=x[index]

                ; can't interpolate?
                if (x[0] gt x0 or x[-1] lt x0) then begin
                    ; edge value
                    if (x[0] gt x0) then a=y[0] else a=y[-1]
                endif else a=interpol_wrap(y,x,x0,/spline,/quad)
                f[ilgeps,iteff,ilgg]=a
            endfor ; lgeps
        endfor ; teff
    endfor ; lgg
    f0=f

    ; log(g)
    f=dblarr(grid.nlgeps,grid.nteff)
    f[*,*]=minval
    for iteff=0l,grid.nteff-1l do begin
        for ilgeps=0l,grid.nlgeps-1l do begin
            y=f0[ilgeps,iteff,*]
            y=reform(y)
            x=grid.lgg
            x0=star.lgg

            index=where(y gt lowval,/null,count)
            if count eq 0l then continue
            y=y[index]
            x=x[index]

            ; can't interpolate?
            if (x[0] gt x0 or x[-1] lt x0) then begin
                ; edge value
                if (x[0] gt x0) then a=y[0] else a=y[-1]
            endif else a=interpol_wrap(y,x,x0,/spline,/quad)
            f[ilgeps,iteff]=a
        endfor ; lgeps
    endfor ; teff
    f0=f

    ; teff
    f=dblarr(grid.nlgeps)
    f[*]=minval
    for ilgeps=0l,grid.nlgeps-1l do begin
        y=f0[ilgeps,*]
        y=reform(y)
        x=grid.teff
        x0=star.teff

        index=where(y gt lowval,/null,count)
        if count eq 0l then continue
        y=y[index]
        x=x[index]

        ; can't interpolate?
        if (x[0] gt x0 or x[-1] lt x0) then begin
            ; edge value
            if (x[0] gt x0) then a=y[0] else a=y[-1]
        endif else a=interpol_wrap(y,x,x0,/spline,/quad)
        f[ilgeps]=a
    endfor ; lgeps
    f0=f


    ; lgeps
    y=f0[*]
    y=reform(y)
    x=grid.lgeps
    x0=star.lgeps

    index=where(y gt lowval,/null,count)
    if count eq 0l then stop
    y=y[index]
    x=x[index]

    ; can't interpolate?
    ;if (x[0] gt x0 or x[-1] lt x0) then begin
    ;    ; edge value
    ;    if (x[0] gt x0) then a=y[0] else a=y[-1]
    ;endif else a=interpol_wrap(y,x,x0,/spline,/quad)
    a=interpol_wrap(y,x,x0,/spline,/quad)
    f=a
    f0=f

    return,f0
end
